@@ -12,7 +12,7 @@ try:
 except:
     pass
 
-version = '0.113'
+version = '0.118'
 
 import re
 import tempfile
@@ -1343,13 +1343,11 @@ if is_testing():
 # In[ ]:
 
 
-def imAddTextHeader(img, text, h=None, size=None):
+def imAddTextHeader(img, text, h=None):
     from PIL import Image, ImageDraw, ImageFont
     
     if h is None:
         h = int(np.ceil(img.shape[0] * 0.1))
-    if size is None:
-        size = h
     
     text_img = np.ones((h, img.shape[1], img.shape[2]), dtype=img.dtype) * 255
     im = Image.fromarray(text_img)
@@ -1372,7 +1370,6 @@ def imAddTextHeader(img, text, h=None, size=None):
 
 if is_testing():
     imshow(imAddTextHeader(imNewBlack(100, 100), 'black image'))
-    imshow(imAddTextHeader(imNewBlack(100, 100), 'black image'))
 
 
 # In[ ]:
@@ -1443,11 +1440,17 @@ def ptT(img_np_uint8):
     assert img.dtype == np.uint8, img.dtype
     return torch.Tensor(np.expand_dims(img.transpose([2, 0, 1]), 0) / 255.0).to(torch.float32)
 
-def ptUint8(img_tensor):
+def ptUint8(img_tensor, only_first=True):
+    if isinstance(img_tensor, np.ndarray):
+        return img_tensor
+    
+    if only_first and len(img_tensor.shape) == 4:
+        img_tensor = img_tensor[0]
+    
     import torch
     img = img_tensor
-    assert img.dtype == torch.float32, img.dtype
-    return np.clip(img.detach().cpu().numpy()[0].transpose([1, 2, 0]) * 255, 0, 255).astype(np.uint8)
+    assert isinstance(img, torch.Tensor) and torch.is_floating_point(img), img.dtype
+    return np.clip(img.detach().cpu().numpy().transpose([1, 2, 0]) * 255, 0, 255).astype(np.uint8)
     
 
 
@@ -1459,6 +1462,13 @@ if is_testing():
     t = ptT(iOrig)
     i = ptUint8(t)
     print('iOrig', np.min(iOrig), np.max(iOrig), iOrig.shape, 't', t.min(), t.max(), t.shape, 'i', i.min(), i.max(), i.shape, 'err', np.mean(np.abs(iOrig - i)))
+    
+    # Always take the first axis
+    assert all(ptUint8(t).ravel() == ptUint8(t[0]).ravel())
+    
+    # Do nothing if already np
+    assert all(ptUint8(ptUint8(t)).ravel() == ptUint8(t).ravel())
+    
 
 
 # # Files
@@ -1704,10 +1714,11 @@ def jnImgToHtmlB64(img):
     img_tag = '<img src="data:image/png;base64,{0}" style="display:block; margin:0 auto;">'.format(imgB64)
     return img_tag
 
-#def jnHtmlImgsRow(imgs, names=None):
+def jnImageRow(*args, **kwargs):
+    return jnImageGallery(*args, **kwargs)
 
-def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, title=None, n_cols=None):
-    if isinstance(imgs, OrderedDict):
+def jnImageGallery(imgs, names=None, width=100, external=False, title_img=None, title=None, n_cols=None, as_html=False, text_size=12):
+    if isinstance(imgs, dict):
         names, imgs = list(zip(*imgs.items()))
     
     if n_cols is None:
@@ -1717,7 +1728,7 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
         n_rows = int(np.ceil(len(imgs) / n_cols))
     
     _row = lambda x: '<tr>' + x + '</tr>'
-    _col = lambda x: '<td style="text-align:center; word-wrap:break-word;">' + x + '</td>'
+    _col = lambda x: f'<td style="text-align:center; word-wrap:break-word"><strong style="font-size: {text_size}px;">' + x + '</strong></td>'
     _nacols = lambda x: prefix_name + "".join(map(_col, x))
     
     prefix_name = "" if title is None else _col(title)
@@ -1750,6 +1761,10 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
         html += html_header + html_imgs
     
     html = _tab(html)
+    
+    if as_html:
+        return html
+    
     display(HTML(html))
 
 
@@ -1758,6 +1773,7 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
 
 if is_testing():
     jnImageRow(imgs=[imNewBlack(20, 20), imNewBlack(40, 20), imNewBlack(20, 800)], names=["longWordNoSpace"*10, "b", "c"], n_cols=None)
+    jnImageRow(imgs=[imNewBlack(20, 20), imNewBlack(40, 20), imNewBlack(20, 800)], names=["longWordNoSpace"*10, "b", "c"], n_cols=None, text_size=5)
     jnImageRow(imgs=[imNewBlack(20, 20), imNewBlack(40, 20), imNewBlack(20, 800)], names=["longWordNoSpace"*10, "b", "c"], n_cols=2)
    
 
@@ -1984,7 +2000,6 @@ if is_testing():
     if output.strip() != version:
         raise RuntimeError(output.strip(), 'Version correct: ', output.strip() == version, "(", output.strip(), version, ")")
     
-    
 
 
 # In[ ]:
@@ -2043,5 +2058,5 @@ if is_testing():
 # In[ ]:
 
 
-0
+
 
